# Common functionality between Unix connect logic and local GUITCL mode start
package require -exact comm 4.2.1

global ixAppPluginManager

# cleanup from old package require
if { [info exists ixAppPluginManager] } {
    unset ixAppPluginManager
}

namespace eval ::IxLoadPrivate:: {
    variable HostIP "";           # the IP address of the remote host
    variable ChannelId "";        # the channel ID after we are connected
    variable TempChannelId "";    # the possible channel ID while we are connecting
    variable NextCommand 0;       # counter to create local commands automatically
    # Local Commands holds a list of commands that are interpreted locally rather than being sent
    # across to the remote scripting service for execution.
    variable LocalCommands {connect disconnect retieveFile}
    namespace export {[a-z]*}
}

# a namespace to exactly mirror the tp namespace on the IxLoad remote scripting service box
namespace eval ::tp:: {
}

proc ::IxLoadPrivate::getChannelId {} {
    variable ChannelId
    return $ChannelId
}

###########################################################################
#
# The most basic send command. This requires that args be a list.
#
# Executes the list argument on the remote machine
#
###########################################################################
proc ::IxLoadPrivate::Send {args} {
    variable ChannelId
    #puts "Sending: $args"
    return [ ::comm::comm send "$ChannelId" [lindex $args 0] ]
}



###########################################################################
###########################################################################
proc ::IxLoadPrivate::createLocalProxyCommand {args} {
    variable NextCommand
    set localCommandName "::IxLoadPrivate::_rtp$NextCommand"
    set NextCommand [expr $NextCommand + 1]
    set localCommandBody [format {{::IxLoadPrivate::Send [concat %s $args]}} $args]
    set localCommand "proc $localCommandName {args} $localCommandBody"
    eval $localCommand
    return $localCommandName
}


###########################################################################
#
# expands args and then Sends the command to the remoteScriptingService
# for execution
#
###########################################################################
proc ::IxLoadPrivate::remoteCommand {args} {
     set command [format "::IxLoad %s" $args]
     return [ Send $command ]
}

###########################################################################
#
# Remote proxy for the IxLoad new command.
#
###########################################################################
proc ::IxLoadPrivate::new {args} {
    set remoteResult [Send [concat ::IxLoad new $args]]
    if { [lsearch $remoteResult ::tp::*] < 0 } {
	error $remoteResult
    } else {
	set result [createLocalProxyCommand $remoteResult]
    }
    return $result
}

###########################################################################
#
# Remote proxy for the IxLoad delete command.
#
###########################################################################
proc ::IxLoadPrivate::delete {args} {
    if { [lsearch $args ::tp::*] != 1 } {
	error "Invalid input to delete: $args. Must be a value returned by ::IxLoad new"
    } else {
	remoteCommand "delete [lindex $args 1]"
    }
}

###########################################################################
#
# Internal command to define a remote array such that array references
# on arrays so defined are resolved remotely by IxLoad.
#
###########################################################################

proc ::IxLoadPrivate::remoteArray {arrayName} {
    array set $arrayName {}
    trace variable $arrayName r ::IxLoadPrivate::arrayTrace
}

::IxLoadPrivate::remoteArray ::ixLogger
::IxLoadPrivate::remoteArray ::ixIP
::IxLoadPrivate::remoteArray ::ixClientNetwork
::IxLoadPrivate::remoteArray ::ixServerNetwork
::IxLoadPrivate::remoteArray ::ixNetworkGroup
::IxLoadPrivate::remoteArray ::ixIpDistributionGroup
::IxLoadPrivate::remoteArray ::ixNetworkRange
::IxLoadPrivate::remoteArray ::ixBandwidthLimit
::IxLoadPrivate::remoteArray ::ixEmulatedRouterIpAddressRange
::IxLoadPrivate::remoteArray ::ixCard
::IxLoadPrivate::remoteArray ::ixAgent
::IxLoadPrivate::remoteArray ::ixTestController
::IxLoadPrivate::remoteArray ::ixTcpParameters
::IxLoadPrivate::remoteArray ::ixTcpParametersFull
::IxLoadPrivate::remoteArray ::ixImpairment
::IxLoadPrivate::remoteArray ::ixIpsec
::IxLoadPrivate::remoteArray ::ixPppoe
::IxLoadPrivate::remoteArray ::ixDhcp
::IxLoadPrivate::remoteArray ::ixDhcpOptions
::IxLoadPrivate::remoteArray ::ixDhcpPD
::IxLoadPrivate::remoteArray ::ixDhcpPDClient
::IxLoadPrivate::remoteArray ::ixDns
::IxLoadPrivate::remoteArray ::_ixDnsServerItem
::IxLoadPrivate::remoteArray ::_ixDnsSuffixItem
::IxLoadPrivate::remoteArray ::ixDut
::IxLoadPrivate::remoteArray ::ixDutConfigSLB
::IxLoadPrivate::remoteArray ::ixDutConfigVip
::IxLoadPrivate::remoteArray ::ixDutConfigVirtual
::IxLoadPrivate::remoteArray ::ixIntRange
::IxLoadPrivate::remoteArray ::ixDutNetworkRange
::IxLoadPrivate::remoteArray ::ixDutProtocolPortRange
::IxLoadPrivate::remoteArray ::ixDutSlbRange
::IxLoadPrivate::remoteArray ::ixViewOptions
::IxLoadPrivate::remoteArray ::ixViewFilter
::IxLoadPrivate::remoteArray ::ixViewProtocolFilter
::IxLoadPrivate::remoteArray ::ixViewCustomFilter
::IxLoadPrivate::remoteArray ::ixScriptGen

::IxLoadPrivate::remoteArray ::ixPort
::IxLoadPrivate::remoteArray ::ixPortMap
::IxLoadPrivate::remoteArray ::ixStackMode
::IxLoadPrivate::remoteArray ::ixTimeline

::IxLoadPrivate::remoteArray ::ixAdvancedIteration
::IxLoadPrivate::remoteArray ::ixLinearTimeSegment
::IxLoadPrivate::remoteArray ::ixRepeatingTimeSegment
::IxLoadPrivate::remoteArray ::ixPoissonTimeSegment
::IxLoadPrivate::remoteArray ::ixStepsTimeSegment
::IxLoadPrivate::remoteArray ::ixBurstsTimeSegment
::IxLoadPrivate::remoteArray ::ixPulsesTimeSegment

::IxLoadPrivate::remoteArray ::ixMatchLongestTimeline
::IxLoadPrivate::remoteArray ::ixObjective

::IxLoadPrivate::remoteArray ::ixServerTrafficNetworkMapping
::IxLoadPrivate::remoteArray ::ixClientTrafficNetworkMapping
::IxLoadPrivate::remoteArray ::ixArpSettings
::IxLoadPrivate::remoteArray ::ixRepository
::IxLoadPrivate::remoteArray ::ixSubmap
::IxLoadPrivate::remoteArray ::ixTrafficFlow
::IxLoadPrivate::remoteArray ::ixTrafficColumn
::IxLoadPrivate::remoteArray ::ixNetTraffic
::IxLoadPrivate::remoteArray ::ixSubscriberNetTraffic
::IxLoadPrivate::remoteArray ::ixActivityParamaters
::IxLoadPrivate::remoteArray ::ixNetworkActivity
::IxLoadPrivate::remoteArray ::ixNetworkCommand

::IxLoadPrivate::remoteArray ::ixScenarioElementFactory
::IxLoadPrivate::remoteArray ::ixNetworkType
::IxLoadPrivate::remoteArray ::ixActivityFunction
::IxLoadPrivate::remoteArray ::ixScenarioElementType

::IxLoadPrivate::remoteArray ::RealFileObject
::IxLoadPrivate::remoteArray ::FTP_Client
::IxLoadPrivate::remoteArray ::ixFtpAction
::IxLoadPrivate::remoteArray ::FTP_Server
::IxLoadPrivate::remoteArray ::HTTP_Client
::IxLoadPrivate::remoteArray ::ixHttpHeaderString
::IxLoadPrivate::remoteArray ::ixHttpResponseString
::IxLoadPrivate::remoteArray ::ixHttpAction
::IxLoadPrivate::remoteArray ::HTTP_Server
::IxLoadPrivate::remoteArray ::PageObject
::IxLoadPrivate::remoteArray ::ResponseHeader
::IxLoadPrivate::remoteArray ::CookieObject
::IxLoadPrivate::remoteArray ::ixCookieContent
::IxLoadPrivate::remoteArray ::CustomPayloadObject
::IxLoadPrivate::remoteArray ::RTSP_Client
::IxLoadPrivate::remoteArray ::RtspHeaders
::IxLoadPrivate::remoteArray ::RtspHeader
::IxLoadPrivate::remoteArray ::RtspCommand
::IxLoadPrivate::remoteArray ::PresentationItem
::IxLoadPrivate::remoteArray ::Content
::IxLoadPrivate::remoteArray ::Stream

::IxLoadPrivate::remoteArray ::RTSP_Server
::IxLoadPrivate::remoteArray ::POP3_Client
::IxLoadPrivate::remoteArray ::Pop3Command
::IxLoadPrivate::remoteArray ::POP3_Server
::IxLoadPrivate::remoteArray ::SMTP_Client
::IxLoadPrivate::remoteArray ::SMTP_Server
::IxLoadPrivate::remoteArray ::MailMessage
::IxLoadPrivate::remoteArray ::MailAttachment
::IxLoadPrivate::remoteArray ::MailHeader
::IxLoadPrivate::remoteArray ::SmtpCommand
::IxLoadPrivate::remoteArray ::MailBoxItem

::IxLoadPrivate::remoteArray ::DDoS_Client

::IxLoadPrivate::remoteArray ::ixPropertyClass_File
::IxLoadPrivate::remoteArray ::ixPropertyClass_Folder


############################################################################
#
#  The Primary IxLoad command. All other commands are invoked via:
#  
#  ::IxLoad subcommand args
#
#      supported subcommands:
#
#            connect       "connect to remote IxLoad scripting service"
#            disconnect    "disconnect from scripting service"
#            new           "create and return new instance of IxLoad object"
#            delete        "delete IxLoad object instance returned by new"
#            eval          "evaluate remotely on server"
#  
#  For proxy commands, ::IxLoadPrivate::remoteCommand is used to execute
#  the args.
#
#  For non-proxy commands, we just prefix cmd with the private namespace
#  qualifier and evaluate "cmd args". The implementation of the respective
#  commands is within namespace ::IxLoadPrivate::
#
############################################################################
proc ::IxLoad {subcmd args} {
    if { $subcmd == "eval" } {
        # don't want to define eval proc, so just hook it here
        set command [format "::IxLoadPrivate::Send {%s}" $args]
    } elseif { [lsearch $::IxLoadPrivate::LocalCommands $subcmd] >= 0 } {
        # local-implemented command.
        set command [format "::IxLoadPrivate::%s %s" $subcmd $args ]
    } elseif { $subcmd == "retrieveFileCopy" } {
        if { [llength $args]!=2} {
            error "Usage: ::IxLoad retrieveFileCopy source destination"
        }
        set blocksize 5242880
        set filesize [::IxLoad retrieveFileBlocks [list [lindex $args 0] -1]]
        proc ceil x {expr {ceil($x)}}
        set blocks [ceil [expr 1.0 * $filesize / $blocksize]]
        puts "Please wait while the file are being transferred ..."
        for {set i 0} { $i < $blocks } {incr i} {
            set offset [expr $i * $blocksize]
            set contents [::IxLoad retrieveFileBlocks [list [lindex $args 0] $offset]]
            if {$offset == 0} {
                set fileId [open [lindex $args 1] w]
            } else {
                set fileId [open [lindex $args 1] a+]
            }
            fconfigure $fileId -translation binary
            puts -nonewline $fileId $contents
            close $fileId
        } 
        puts "Transfer completed!"
        return
    } elseif { $subcmd == "sendFileCopy"} {
        if { [llength $args]!=2} {
            error "Usage: ::IxLoad sendFileCopy source destination"
        }
        set f [open [lindex $args 0]]
        fconfigure $f -translation binary
        set contents [read $f]
        close $f
        set fileName [lindex $args 1]
        # Convert the fileName to hex in order to pass any special characters
        binary scan $fileName H* hexFileName
        # Convert the file contents to hex as it might be binary data.
        binary scan $contents H* data
        set command {::IxLoadPrivate::Send [concat ::IxLoad unhexAndSendFile $hexFileName $data]}
    } elseif { $subcmd == "retrieveResults" } {
        if { [llength $args]!=1} {
            error "Usage: ::IxLoad retrieveResults destinationDir"
        }
	    set destDir [lindex $args 0]
	    if { [file type $destDir] != "directory" } {
	        error "::IxLoad retrieveResults requires a directory name for its argument"
	    }
	    set tc [::IxLoad new ixTestController]
	    set resultsDir [$tc getRunResultDirFull]
        regsub -all {\\} $resultsDir {/} resultsDir
	    set numResultPaths [llength [file split $resultsDir]]
	    set resultsContents [::IxLoad clientDir $resultsDir]
	    foreach resultFile $resultsContents {
	        set destPaths [file split $resultFile]
	        set destLength [llength $destPaths]
	        set destLength [expr $destLength - 1]
	        # calculate path from end of resultsDir
	        set destPath [lrange $destPaths $numResultPaths $destLength]
	        set destPath [string trimleft $destPath '{']
	        set destPath [string trimright $destPath '}']
	        set destFile [file join $destDir $destPath]
	        ::IxLoad retrieveFileCopy $resultFile $destFile
	    }
	    return
    } else {
        # remote command (e.g. new delete retrieveFile)
        set command [format "::IxLoadPrivate::Send {::IxLoad %s %s}" $subcmd $args]
    }
    return [eval $command]
}


############################################################################
#
#  Hook the unknown handler with our own, if we haven't done so already
#  (as might be the case with multiple package requires of IxLoad).
#
############################################################################

if { [catch { info args ::IxLoadPriorUnknown }] } {
    # rename prior unknown if not already done
    rename ::unknown ::IxLoadPriorUnknown
}

############################################################################
#
#  Our unknown handler.
#
#  This handler causes all commands of the form ::tp::* to be eval'ed remotely. 
#  Commands of that form are generated on the scripting service side and 
#  correspond to IxLoad objects. This technique saves us from having to
#  generate a proxy for each and every such object (which is fairly difficult 
#  to do correctly).
#
############################################################################

proc ::unknown {args} {
    set cmd [lindex $args 0]
    if { [string match "::tp::*" $cmd] ||
         [string match "*statCollectorUtils::*" $cmd]} {
	# need to pass Send a list
	set command [list ::IxLoadPrivate::Send [concat $args]]
    } else {
	# need it flat
	set command [concat ::IxLoadPriorUnknown $args]
    }
    uplevel $command
}

# if { [regexp "^((::tp::*)|((::)?statCollectorUtils*))" $cmd] }